import { ResolvedConfigContext } from "@modern-js/core";
import { DEFAULT_DEV_HOST } from "@modern-js/utils";
import { printInstructions } from "../utils/printInstructions";
import { setServer, createServer, injectDataLoaderPlugin } from "../utils/createServer";
import { generateRoutes } from "../utils/routes";
import { buildServerConfig } from "../utils/config";
import { getServerInternalPlugins } from "../utils/getServerInternalPlugins";
const dev = async (api, options, devServerOptions = {}) => {
  var _normalizedConfig_tools;
  if (options.analyze) {
    process.env.BUNDLE_ANALYZE = "true";
  }
  let normalizedConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const hookRunners = api.useHookRunners();
  normalizedConfig = {
    ...normalizedConfig,
    cliOptions: options
  };
  ResolvedConfigContext.set(normalizedConfig);
  const { appDirectory, distDirectory, port, apiOnly, serverConfigFile, metaName } = appContext;
  await buildServerConfig({
    appDirectory,
    distDirectory,
    configFile: serverConfigFile,
    watch: true
  });
  await hookRunners.beforeDev();
  if (!appContext.builder && !apiOnly) {
    throw new Error("Expect the Builder to have been initialized, But the appContext.builder received `undefined`");
  }
  await generateRoutes(appContext);
  const serverInternalPlugins = await getServerInternalPlugins(api);
  const serverOptions = {
    dev: {
      port,
      https: normalizedConfig.dev.https,
      host: normalizedConfig.dev.host,
      ...(_normalizedConfig_tools = normalizedConfig.tools) === null || _normalizedConfig_tools === void 0 ? void 0 : _normalizedConfig_tools.devServer
    },
    appContext: {
      metaName,
      appDirectory: appContext.appDirectory,
      sharedDirectory: appContext.sharedDirectory,
      apiDirectory: appContext.apiDirectory,
      lambdaDirectory: appContext.lambdaDirectory
    },
    pwd: appDirectory,
    config: normalizedConfig,
    serverConfigFile,
    internalPlugins: injectDataLoaderPlugin(serverInternalPlugins),
    ...devServerOptions
  };
  if (apiOnly) {
    var _normalizedConfig_dev;
    const app = await createServer({
      ...serverOptions,
      compiler: null
    });
    const host = ((_normalizedConfig_dev = normalizedConfig.dev) === null || _normalizedConfig_dev === void 0 ? void 0 : _normalizedConfig_dev.host) || DEFAULT_DEV_HOST;
    app.listen({
      port,
      host
    }, async (err) => {
      if (err) {
        throw err;
      }
      printInstructions(hookRunners, appContext, normalizedConfig);
    });
  } else {
    const { server } = await appContext.builder.startDevServer({
      serverOptions
    });
    setServer(server);
  }
};
export {
  dev
};
