import { logger, isApiOnly, getTargetDir } from "@modern-js/utils";
import server from "@modern-js/prod-server";
import { printInstructions } from "../utils/printInstructions";
import { injectDataLoaderPlugin } from "../utils/createServer";
import { getServerInternalPlugins } from "../utils/getServerInternalPlugins";
const start = async (api) => {
  var _userConfig_source, _userConfig_output_distPath;
  const appContext = api.useAppContext();
  const userConfig = api.useResolvedConfigContext();
  const hookRunners = api.useHookRunners();
  const { appDirectory, port, serverConfigFile, metaName } = appContext;
  logger.info(`Starting production server...`);
  const apiOnly = await isApiOnly(appContext.appDirectory, userConfig === null || userConfig === void 0 ? void 0 : (_userConfig_source = userConfig.source) === null || _userConfig_source === void 0 ? void 0 : _userConfig_source.entriesDir, appContext.apiDirectory);
  const serverInternalPlugins = await getServerInternalPlugins(api);
  const app = await server({
    pwd: appDirectory,
    config: {
      ...userConfig,
      dev: userConfig.dev,
      output: {
        path: (_userConfig_output_distPath = userConfig.output.distPath) === null || _userConfig_output_distPath === void 0 ? void 0 : _userConfig_output_distPath.root,
        ...userConfig.output || {}
      }
    },
    appContext: {
      metaName,
      sharedDirectory: getTargetDir(appContext.sharedDirectory, appContext.appDirectory, appContext.distDirectory),
      apiDirectory: getTargetDir(appContext.apiDirectory, appContext.appDirectory, appContext.distDirectory),
      lambdaDirectory: getTargetDir(appContext.lambdaDirectory, appContext.appDirectory, appContext.distDirectory)
    },
    serverConfigFile,
    internalPlugins: injectDataLoaderPlugin(serverInternalPlugins),
    apiOnly
  });
  app.listen(port, async (err) => {
    if (err) {
      throw err;
    }
    await printInstructions(hookRunners, appContext, userConfig);
  });
};
export {
  start
};
